// -*- C++ -*-
/*!
 * @file  ARMarkerDetectTest.cpp
 * @brief Test module for AR Marker Detection
 * @date $Date$
 *
 * @author Tsuyoshi Suenaga, NAIST
 * tsuyo-s@is.naist.jp
 *
 * GNU General Public License
 *
 * $Id$
 */

#include "ARMarkerDetectTest.h"

// Module specification
// <rtc-template block="module_spec">
static const char* armarkerdetecttest_spec[] =
  {
    "implementation_id", "ARMarkerDetectTest",
    "type_name",         "ARMarkerDetectTest",
    "description",       "Test module for AR Marker Detection",
    "version",           "0.2.0",
    "vendor",            "Tsuyoshi Suenaga, NAIST",
    "category",          "test",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.detect_marker_id", "0",
    // Widget
    "conf.__widget__.detect_marker_id", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ARMarkerDetectTest::ARMarkerDetectTest(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_markerPoseAllIn("MarkerPoseAll", m_markerPoseAll),
    m_markerPoseIn("MarkerPose", m_markerPose),
    m_ObjectRecognitionPortPort("ObjectRecognitionPort")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
ARMarkerDetectTest::~ARMarkerDetectTest()
{
}



RTC::ReturnCode_t ARMarkerDetectTest::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("MarkerPoseAll", m_markerPoseAllIn);
  addInPort("MarkerPose", m_markerPoseIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_ObjectRecognitionPortPort.registerConsumer("RecognitionService", "RecognitionService", m_RecognitionService);
  
  // Set CORBA Service Ports
  addPort(m_ObjectRecognitionPortPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("detect_marker_id", m_detect_marker_id, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ARMarkerDetectTest::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ARMarkerDetectTest::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ARMarkerDetectTest::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ARMarkerDetectTest::onActivated(RTC::UniqueId ec_id)
{
  PortServiceList* portlist = this->get_ports();
  port_connect_check = new bool [portlist->length()];
  for(unsigned int i=0; i<portlist->length(); ++i){
    PortService_ptr port;
    port = (*portlist)[i];
    if(port->get_port_profile()->connector_profiles.length()!=0){
      port_connect_check[i]=true;
    }else{
      port_connect_check[i]=false;
    }
  }

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ARMarkerDetectTest::onDeactivated(RTC::UniqueId ec_id)
{
  delete port_connect_check;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ARMarkerDetectTest::onExecute(RTC::UniqueId ec_id)
{
  // サービスポート接続状態のチェック
  if(port_connect_check[2]==true){
    m_RecognitionService->setModelID(m_detect_marker_id);
    fprintf(stderr, "ID was set to %ld, [%ld]\n",
            m_detect_marker_id, m_RecognitionService->getModelID());
    RTC_DEBUG(("ID was set to %ld, [%ld]",
               m_detect_marker_id, m_RecognitionService->getModelID()));
  }

  if (m_markerPoseAllIn.isNew()) {
    m_markerPoseAllIn.read();
    fprintf(stderr, "Total of detected marker is %ld\n",
            m_markerPoseAll.data.length());
    RTC_DEBUG(("Total of detected marker is %d",
               m_markerPoseAll.data.length()));
    for (unsigned int n = 0; n < m_markerPoseAll.data.length(); ++n) {
      fprintf(stderr, "Detected marker [%ld]\n",
              m_markerPoseAll.data[n].id);
      RTC_DEBUG(("Marker ID [%ld] is detected",
                 m_markerPoseAll.data[n].id));
      if (m_markerPoseAll.data[n].pose.length() != 12) {
        fprintf(stderr, "  Data size is wrong. Pose should have 12 params.");
        RTC_WARN(("Data size is wrong. Pose should have 12 params"));
        continue;
      }
      RTC_TRACE(("Pose of marker ID [%ld]",
                 m_markerPoseAll.data[n].id));
      for (unsigned int i = 0; i < 3; ++i) {
        std::string comment = " ";
        for (unsigned int j = 0; j < 4; ++j) {
          fprintf(stderr, "  %f",
                  m_markerPoseAll.data[n].pose[i*4 + j]);
          // RTC::Loggerでの位置姿勢ログ用
          char s[32];
          snprintf(s, 32, " %12.6lf",
                   m_markerPoseAll.data[n].pose[i*4 + j]);
          comment = comment + " " + s;
        }
        fprintf(stderr, "\n");      
        RTC_TRACE(("%s", comment.c_str()));
      }
    }
  }

  if (m_markerPoseIn.isNew()) {
    m_markerPoseIn.read();
    if (m_markerPose.data.length() != 12) {
      return RTC::RTC_ERROR;
    }
    fprintf(stderr, "Specified marker [%ld]\n", m_detect_marker_id);
    RTC_DEBUG(("Specified Marker ID [%ld] is detected",
               m_detect_marker_id));
    RTC_TRACE(("Pose of specified marker ID [%ld]",
               m_detect_marker_id));
    for (unsigned int i = 0; i < 3; ++i) {
      std::string comment = " ";
      for (unsigned int j = 0; j < 4; ++j) {
        fprintf(stderr, " %f",
                m_markerPose.data[i*4 + j]);
        // RTC::Loggerでの位置姿勢ログ用
        char s[32];
        snprintf(s, 32, " %12.6lf",
                 m_markerPose.data[i*4 + j]);
        comment = comment + " " + s;
      }
      fprintf(stderr, "\n");      
      RTC_TRACE(("%s", comment.c_str()));
    }
  }

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ARMarkerDetectTest::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ARMarkerDetectTest::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ARMarkerDetectTest::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ARMarkerDetectTest::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ARMarkerDetectTest::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ARMarkerDetectTestInit(RTC::Manager* manager)
  {
    coil::Properties profile(armarkerdetecttest_spec);
    manager->registerFactory(profile,
                             RTC::Create<ARMarkerDetectTest>,
                             RTC::Delete<ARMarkerDetectTest>);
  }
  
};


